<?php

namespace Gridly;

use Gridly\Column\DecoratorPipeline;
use Gridly\Filter\Filter;
use Gridly\Paginator\Paginator;
use Gridly\Source\AbstractSource;

class Grid
{
    /**
     * @var AbstractSource
     */
    private $source;

    /**
     * @var Paginator
     */
    private $paginator;

    /**
     * @var array
     */
    private $filters;

    /**
     * @var array
     */
    private $columnDecorators;

    /**
     * @param AbstractSource $source
     * @param Paginator $paginator
     */
    public function __construct(AbstractSource $source, Paginator $paginator)
    {
        $this->paginator = $paginator;
        $this->source = $source;
        $this->columnDecorators = [];
        $this->filters = [];
    }

    /**
     * @param int $count
     * @return Grid
     */
    public function setItemsPerPage(int $count = 10): Grid
    {
        $this->paginator->setItemsPerPage($count);
        return $this;
    }

    /**
     * @param int|null $page
     * @param array|null $hiddenColumns
     * @return ResultSet
     */
    public function getPageItems(?int $page = 1, ?array $hiddenColumns = []): ResultSet
    {
        $this->source->applyFilters($this->filters);

        return new ResultSet(
            $this->paginator->getPageItems($page),
            $hiddenColumns,
            $this->columnDecorators
        );
    }

    /**
     * @param Filter $filter
     * @return Grid
     */
    public function addFilter(Filter $filter): Grid
    {
        $this->filters[] = $filter;
        return $this;
    }

    /**
     * @param string $columnName
     * @param callable $decorator
     * @return Grid
     */
    public function addColumnDecorator(string $columnName, callable $decorator): self
    {
        if (!isset($this->columnDecorators[$columnName])) {
            $this->columnDecorators[$columnName] = new DecoratorPipeline();
        }

        /** @var DecoratorPipeline $pipeline */
        $pipeline = $this->columnDecorators[$columnName];
        $pipeline->pipe($decorator);

        return $this;
    }

    /**
     * @return int
     */
    public function getTotalItems(): int
    {
        return $this->source->count();
    }
}
