<?php

namespace Gridly;

use Gridly\Column\Column;
use Gridly\Column\Decorator;
use Traversable;

class ResultSet implements \Countable, \IteratorAggregate
{
    /** @var iterable  */
    private $rawData;

    /** @var array  */
    private $rows;

    /** @var array  */
    private $headers;

    /** @var array */
    private $hiddenColumns;

    /** @var array */
    private $columnDecorators;

    /**
     * @param iterable $data
     * @param array $hiddenColumns
     * @param array $columnDecorators
     */
    public function __construct(
        iterable $data,
        array $hiddenColumns = [],
        array $columnDecorators = []
    ) {
        $this->rawData = $data;
        $this->rows = [];
        $this->headers = [];
        $this->hiddenColumns = $hiddenColumns;
        $this->columnDecorators = $columnDecorators;
        $this->processData();
    }

    /**
     * @return array
     */
    public function getHeaders(): array
    {
        return $this->headers;
    }

    /**
     * @return array
     */
    public function getRows(): array
    {
        return $this->rows;
    }

    /**
     * @return int
     */
    public function count(): int
    {
        return count($this->rows);
    }

    /**
     * @return \ArrayIterator|Traversable
     */
    public function getIterator()
    {
        return new \ArrayIterator($this->rows);
    }

    private function processData(): void
    {
        foreach ($this->rawData as $i => $entry) {
            $row = [];
            foreach ($entry as $name => $value) {
                if (!\in_array($name, $this->hiddenColumns)) {
                    if ($i === 0) {
                        $this->headers[] = $name;
                    }

                    $column = new Column($name, $value);
                    if (isset($this->columnDecorators[$column->name()])) {
                        /** @var Decorator $decorator */
                        $decorator = $this->columnDecorators[$column->name()];
                        $column = $decorator($column);
                    }

                    $row[$name] = $column;
                }
            }

            $this->rows[] = $row;
        }
    }
}
