<?php

namespace Gridly\Source;

use Gridly\Filter\Filter;
use Gridly\SourceException;

class Pdo extends AbstractSource
{
    /** @var \PDO  */
    private $pdo;

    /** @var string */
    private $tableName;

    /** @var string */
    private $query;

    /** @var array */
    private $queryParams;

    /**
     * @param \PDO $pdo
     * @param string $tableName
     */
    public function __construct(\PDO $pdo, string $tableName)
    {
        $this->pdo = $pdo;
        $this->tableName = '`' . str_replace('`', '``', $tableName) . '`';
        $this->query = 'SELECT * FROM ' . $this->tableName;
        $this->queryParams = [];
    }

    /**
     * @param array|null $filters
     * @return AbstractSource
     * @throws SourceException
     */
    public function applyFilters(?array $filters = []): AbstractSource
    {
        if (empty($filters)) {
            return $this;
        }

        $this->query .= ' WHERE ';

        $filtersAmount = count($filters);
        $lastFilterIndex = $filtersAmount - 1;

        /** @var Filter $filter */
        foreach ($filters as $i => $filter) {
            switch ($filter->getOperand()) {
                case Filter::OP_EQUAL:
                    $this->query .= $filter->getColumnName() . ' = ?';
                    $this->queryParams[] = $filter->getValue();
                    break;
                case Filter::OP_LIKE:
                    $value = '%' . $filter->getValue() . '%';
                    $this->query .= $filter->getColumnName() . ' LIKE ?';
                    $this->queryParams[] = $value;
                    break;
                default:
                    SourceException::onUnsupportedFilterOperand(
                        $filter->getOperand(),
                        self::class
                    );
            }

            if ($filtersAmount > 1 && $i !== $lastFilterIndex) {
                $this->query .= ' AND ';
            }
        }

        return $this;
    }

    /**
     * @param int $offset
     * @param int $itemCountPerPage
     * @return iterable
     */
    public function getItems($offset, $itemCountPerPage): iterable
    {
        $limitStatement = ' LIMIT ';
        $limitStatement .= $offset . ', ' . $itemCountPerPage;

        $this->query .= $limitStatement;

        /** @var \PDOStatement $stmt */
        $stmt = $this->pdo->prepare($this->query);
        $stmt->execute($this->queryParams);

        return $stmt->fetchAll();
    }

    /**
     * @return int
     */
    public function count(): int
    {
        $query = 'SELECT COUNT(*) FROM ' . $this->tableName;
        return $this->pdo->query($query)->fetchColumn();
    }
}
