<?php

namespace GridlyTest;

use Gridly\Grid;
use Gridly\Paginator\Paginator;
use Gridly\ResultSet;
use Gridly\Source\AbstractSource;
use PHPUnit\Framework\TestCase;

class ResultSetTest extends TestCase
{
    /**
     * @throws \ReflectionException
     */
    public function testGridLoadReturnResultSet()
    {
        $grid = $this->createGrid();

        $result = $grid->getPageItems();
        $this->assertInstanceOf(ResultSet::class, $result);
    }

    /**
     * @throws \ReflectionException
     */
    public function testNotEmptyResultSetCount()
    {
        $grid = $this->createGrid();

        $result = $grid->getPageItems();
        $this->assertEquals($result->count(), 2);
    }

    /**
     * @throws \ReflectionException
     */
    public function testEmptyResultSetCount()
    {
        $grid = $this->createGrid();

        $result = $grid->getPageItems();
        $this->assertEquals($result->count(), 2);
    }

    /**
     * @return Grid
     * @throws \ReflectionException
     */
    protected function createGrid(): Grid
    {
        $paginatorMock = $this->getMockBuilder(Paginator::class)
            ->setMethods(['getSource', 'getPageItems', 'setItemsPerPage'])
            ->disableOriginalConstructor()
            ->getMock();

        $paginatorMock
            ->expects($this->once())
            ->method('getPageItems')
            ->will($this->returnValue(
                [
                    [
                        'id' => 1,
                        'name' => 'Aaron Turmonen',
                        'email' => 'aaron@example.com'
                    ],
                    [
                        'id' => 2,
                        'name' => 'Lewis Skorski',
                        'email' => 'lewis@example.com'
                    ]
                ]
            ));

        $sourceMock = $this->createMock(AbstractSource::class);

        return new Grid($sourceMock, $paginatorMock);
    }
}
